@@ -92,7 +92,13 @@ static void tlshd_client_anon_handshake(struct tlshd_handshake_parms *parms)
 	ret = gnutls_set_default_priority(session);
 	if (ret != GNUTLS_E_SUCCESS) {
 		tlshd_log_gnutls_error(ret);
-		return;
+		goto out_free_creds;
+	}
+
+	ret = tlshd_gnutls_priority_set(session, parms, 0);
+	if (ret != GNUTLS_E_SUCCESS) {
+		tlshd_log_gnutls_error(ret);
+		goto out_free_creds;
 	}
 
 	gnutls_session_set_verify_cert(session, parms->peername, 0);
@@ -106,16 +112,18 @@ out_free_creds:
 }
 
 static gnutls_privkey_t tlshd_privkey;
-static gnutls_pcert_st tlshd_cert;
+static unsigned int tlshd_certs_len = TLSHD_MAX_CERTS;
+static gnutls_pcert_st tlshd_certs[TLSHD_MAX_CERTS];
 
 /*
- * XXX: After this point, tlshd_cert should be deinited on error.
+ * XXX: After this point, tlshd_certs should be deinited on error.
  */
-static bool tlshd_x509_client_get_cert(struct tlshd_handshake_parms *parms)
+static bool tlshd_x509_client_get_certs(struct tlshd_handshake_parms *parms)
 {
 	if (parms->x509_cert != TLS_NO_CERT)
-		return tlshd_keyring_get_cert(parms->x509_cert, &tlshd_cert);
-	return tlshd_config_get_client_cert(&tlshd_cert);
+		return tlshd_keyring_get_certs(parms->x509_cert, tlshd_certs,
+					       &tlshd_certs_len);
+	return tlshd_config_get_client_certs(tlshd_certs, &tlshd_certs_len);
 }
 
 /*
@@ -125,7 +133,7 @@ static bool tlshd_x509_client_get_privkey(struct tlshd_handshake_parms *parms)
 {
 	if (parms->x509_privkey != TLS_NO_PRIVKEY)
 		return tlshd_keyring_get_privkey(parms->x509_privkey,
-						 tlshd_privkey);
+						 &tlshd_privkey);
 	return tlshd_config_get_client_privkey(&tlshd_privkey);
 }
 
@@ -179,8 +187,8 @@ tlshd_x509_retrieve_key_cb(gnutls_session_t session,
 	if (type != GNUTLS_CRT_X509)
 		return -1;
 
-	*pcert_length = 1;
-	*pcert = &tlshd_cert;
+	*pcert_length = tlshd_certs_len;
+	*pcert = tlshd_certs;
 	*privkey = tlshd_privkey;
 	return 0;
 }
@@ -280,7 +288,7 @@ static void tlshd_client_x509_handshake(struct tlshd_handshake_parms *parms)
 	}
 	tlshd_log_debug("System trust: Loaded %d certificate(s).", ret);
 
-	if (!tlshd_x509_client_get_cert(parms))
+	if (!tlshd_x509_client_get_certs(parms))
 		goto out_free_creds;
 	if (!tlshd_x509_client_get_privkey(parms))
 		goto out_free_creds;
@@ -307,6 +315,11 @@ static void tlshd_client_x509_handshake(struct tlshd_handshake_parms *parms)
 		tlshd_log_gnutls_error(ret);
 		goto out_free_creds;
 	}
+	ret = tlshd_gnutls_priority_set(session, parms, 0);
+	if (ret != GNUTLS_E_SUCCESS) {
+		tlshd_log_gnutls_error(ret);
+		goto out_free_creds;
+	}
 	gnutls_certificate_set_verify_function(xcred,
 					       tlshd_client_x509_verify_function);
 
@@ -363,6 +376,12 @@ static void tlshd_client_psk_handshake_one(struct tlshd_handshake_parms *parms,
 	gnutls_session_set_ptr(session, parms);
 	gnutls_credentials_set(session, GNUTLS_CRD_PSK, psk_cred);
 
+	ret = tlshd_gnutls_priority_set(session, parms, key.size);
+	if (ret != GNUTLS_E_SUCCESS) {
+		tlshd_log_gnutls_error(ret);
+		goto out_free_creds;
+	}
+
 	tlshd_log_debug("start ClientHello handshake");
 	tlshd_start_tls_handshake(session, parms);
 	if (!parms->session_status) {
