@@ -87,7 +87,7 @@ bool tlshd_config_init(const gchar *pathname)
 	tlshd_delay_done = tmp > 0 ? tmp : 0;
 
 	keyrings = g_key_file_get_string_list(tlshd_configuration,
-					      "authentication",
+					      "authenticate",
 					      "keyrings", &length, NULL);
 	if (keyrings) {
 		for (i = 0; i < length; i++) {
@@ -104,29 +104,6 @@ void tlshd_config_shutdown(void)
 	g_key_file_free(tlshd_configuration);
 }
 
-#if HAVE_LINUX_OPENAT2_H
-#include <linux/openat2.h>
-
-#ifndef SYS_openat2
-#define SYS_openat2 __NR_openat2
-#endif
-
-static int tlshd_file_open(const char *pathname)
-{
-	static const struct open_how how = {
-		.flags		= O_RDONLY,
-		.resolve	= RESOLVE_NO_SYMLINKS,
-	};
-
-	return (int)syscall(SYS_openat2, 0, pathname, &how, sizeof(how));
-}
-#else
-static int tlshd_file_open(const char *pathname)
-{
-	return open(pathname, O_RDONLY);
-}
-#endif
-
 /*
  * Expected file attributes
  */
@@ -148,7 +125,7 @@ static bool tlshd_config_read_datum(const char *pathname, gnutls_datum_t *data,
 
 	ret = false;
 
-	fd = tlshd_file_open(pathname);
+	fd = open(pathname, O_RDONLY);
 	if (fd == -1) {
 		tlshd_log_perror("open");
 		goto out;
@@ -207,6 +184,9 @@ bool tlshd_config_get_client_truststore(char **bundle)
 	if (!pathname) {
 		g_error_free(error);
 		return false;
+	} else if (access(pathname, F_OK)) {
+		tlshd_log_debug("client x509.truststore pathname \"%s\" is not accessible", pathname);
+		return false;
 	}
 
 	*bundle = strdup(pathname);
@@ -219,14 +199,16 @@ bool tlshd_config_get_client_truststore(char **bundle)
 }
 
 /**
- * tlshd_config_get_client_cert - Get cert for ClientHello from .conf
- * @cert: OUT: in-memory certificate
+ * tlshd_config_get_client_certs - Get certs for ClientHello from .conf
+ * @certs: OUT: in-memory certificates
+ * @certs_len: IN: maximum number of certs to get, OUT: number of certs found
  *
  * Return values:
  *   %true: certificate retrieved successfully
  *   %false: certificate not retrieved
  */
-bool tlshd_config_get_client_cert(gnutls_pcert_st *cert)
+bool tlshd_config_get_client_certs(gnutls_pcert_st *certs,
+				   unsigned int *certs_len)
 {
 	GError *error = NULL;
 	gnutls_datum_t data;
@@ -238,6 +220,9 @@ bool tlshd_config_get_client_cert(gnutls_pcert_st *cert)
 	if (!pathname) {
 		g_error_free(error);
 		return false;
+	} else if (access(pathname, F_OK)) {
+		tlshd_log_debug("client x509.certificate pathname \"%s\" is not accessible", pathname);
+		return false;
 	}
 
 	if (!tlshd_config_read_datum(pathname, &data, TLSHD_OWNER,
@@ -247,8 +232,8 @@ bool tlshd_config_get_client_cert(gnutls_pcert_st *cert)
 	}
 
 	/* Config file supports only PEM-encoded certificates */
-	ret = gnutls_pcert_import_x509_raw(cert, &data,
-					   GNUTLS_X509_FMT_PEM, 0);
+	ret = gnutls_pcert_list_import_x509_raw(certs, certs_len, &data,
+						GNUTLS_X509_FMT_PEM, 0);
 	free(data.data);
 	if (ret != GNUTLS_E_SUCCESS) {
 		tlshd_log_gnutls_error(ret);
@@ -256,7 +241,8 @@ bool tlshd_config_get_client_cert(gnutls_pcert_st *cert)
 		return false;
 	}
 
-	tlshd_log_debug("Retrieved x.509 certificate from %s", pathname);
+	tlshd_log_debug("Retrieved %u x.509 client certificate(s) from %s",
+			*certs_len, pathname);
 	g_free(pathname);
 	return true;
 }
@@ -281,6 +267,9 @@ bool tlshd_config_get_client_privkey(gnutls_privkey_t *privkey)
 	if (!pathname) {
 		g_error_free(error);
 		return false;
+	} else if (access(pathname, F_OK)) {
+		tlshd_log_debug("client x095.private_key pathname \"%s\" is not accessible", pathname);
+		return false;
 	}
 
 	if (!tlshd_config_read_datum(pathname, &data, TLSHD_OWNER,
@@ -330,6 +319,9 @@ bool tlshd_config_get_server_truststore(char **bundle)
 	if (!pathname) {
 		g_error_free(error);
 		return false;
+	} else if (access(pathname, F_OK)) {
+		tlshd_log_debug("server x509.truststore pathname \"%s\" is not accessible", pathname);
+		return false;
 	}
 
 	*bundle = strdup(pathname);
@@ -342,14 +334,16 @@ bool tlshd_config_get_server_truststore(char **bundle)
 }
 
 /**
- * tlshd_config_get_server_cert - Get cert for ServerHello from .conf
- * @cert: OUT: in-memory certificate
+ * tlshd_config_get_server_certs - Get certs for ServerHello from .conf
+ * @certs: OUT: in-memory certificates
+ * @certs_len: IN: maximum number of certs to get, OUT: number of certs found
  *
  * Return values:
  *   %true: certificate retrieved successfully
  *   %false: certificate not retrieved
  */
-bool tlshd_config_get_server_cert(gnutls_pcert_st *cert)
+bool tlshd_config_get_server_certs(gnutls_pcert_st *certs,
+				   unsigned int *certs_len)
 {
 	GError *error = NULL;
 	gnutls_datum_t data;
@@ -361,6 +355,9 @@ bool tlshd_config_get_server_cert(gnutls_pcert_st *cert)
 	if (!pathname) {
 		g_error_free(error);
 		return false;
+	} else if (access(pathname, F_OK)) {
+		tlshd_log_debug("server x509.certificate pathname \"%s\" is not accessible", pathname);
+		return false;
 	}
 
 	if (!tlshd_config_read_datum(pathname, &data, TLSHD_OWNER,
@@ -370,8 +367,8 @@ bool tlshd_config_get_server_cert(gnutls_pcert_st *cert)
 	}
 
 	/* Config file supports only PEM-encoded certificates */
-	ret = gnutls_pcert_import_x509_raw(cert, &data,
-					   GNUTLS_X509_FMT_PEM, 0);
+	ret = gnutls_pcert_list_import_x509_raw(certs, certs_len, &data,
+						GNUTLS_X509_FMT_PEM, 0);
 	free(data.data);
 	if (ret != GNUTLS_E_SUCCESS) {
 		tlshd_log_gnutls_error(ret);
@@ -379,7 +376,8 @@ bool tlshd_config_get_server_cert(gnutls_pcert_st *cert)
 		return false;
 	}
 
-	tlshd_log_debug("Retrieved x.509 certificate from %s", pathname);
+	tlshd_log_debug("Retrieved %u x.509 server certificate(s) from %s",
+			*certs_len, pathname);
 	return true;
 }
 
@@ -403,6 +401,9 @@ bool tlshd_config_get_server_privkey(gnutls_privkey_t *privkey)
 	if (!pathname) {
 		g_error_free(error);
 		return false;
+	} else if (access(pathname, F_OK)) {
+		tlshd_log_debug("server x509.privkey pathname \"%s\" is not accessible", pathname);
+		return false;
 	}
 
 	if (!tlshd_config_read_datum(pathname, &data, TLSHD_OWNER,
